module cordic(
    clk         ,
    rst_n       ,
    din         ,//0-359
    din_vld     ,
    dout_sin    ,
    dout_cos    ,
    dout_vld        
);

`define ROT0  32'd2949120       //45度*2^16
`define ROT1  32'd1740992       //26.5651度*2^16
`define ROT2  32'd919872        //14.0362度*2^16
`define ROT3  32'd466944        //7.1250度*2^16
`define ROT4  32'd234368        //3.5763度*2^16
`define ROT5  32'd117312        //1.7899度*2^16
`define ROT6  32'd58688         //0.8952度*2^16
`define ROT7  32'd29312         //0.4476度*2^16
`define ROT8  32'd14656         //0.2238度*2^16
`define ROT9  32'd7360          //0.1119度*2^16
`define ROT10 32'd3648          //0.0560度*2^16
`define ROT11 32'd1856          //0.0280度*2^16
`define ROT12 32'd896           //0.0140度*2^16
`define ROT13 32'd448           //0.0070度*2^16
`define ROT14 32'd256           //0.0035度*2^16
`define ROT15 32'd128           //0.0018度*2^16

parameter COS_LM = 16'h9b75;    //K=0.607253*2^16,32'h09b74

input              clk     ;
input              rst_n   ;
input              din_vld ;
input  [ 9-1:0]    din     ;

output signed [ 18-1:0]    dout_sin ;
output signed [ 18-1:0]    dout_cos ;
output               dout_vld ;

reg    signed [ 18-1:0]    dout_sin ;
reg    signed [ 18-1:0]    dout_cos ;
reg                  dout_vld ;

reg signed [18-1:0] x[16:0];
reg signed [18-1:0] y[16:0];
reg signed [26-1:0] z[16:0];

reg [ (17-1):0]  din_vld_ff     ;
reg [ 9-1:0]     din_ff         ;
reg [ 2-1:0]     flag           ;
reg [36-1:0]     flag_ff        ;
reg              dout_vld_ff0   ;

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        din_ff <= 0;
        flag   <= 0;
    end
    else if(din_vld)begin
        if(din<90)begin
            din_ff = din;
            flag   = 0;
        end
        else if(din<180)begin
            din_ff = din-90;
            flag   = 1;
        end
        else if(din<270)begin
            din_ff = din-180;
            flag   = 2;
        end
        else begin
            din_ff = din-270;
            flag   = 3;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[0]   <= 0;
        y[0]   <= 0;
        z[0]   <= (din<<16);
    end
    else if(din_vld_ff[0]) begin
        x[0]   <= {2'b0,COS_LM};
        y[0]   <= 0;
        z[0]   <= {1'b0,din_ff,16'b0};
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[1]   <= 0;
        y[1]   <= 0;
        z[1]   <= 0;
    end
    else if(din_vld_ff[1])begin
        if(z[0][25]==0)begin
            x[1] <= x[0] - (y[0]>>>0);
            y[1] <= y[0] + (x[0]>>>0);
            z[1] <= z[0] - `ROT0;
        end
        else begin
            x[1] <= x[0] + (y[0]>>>0);
            y[1] <= y[0] - (x[0]>>>0);
            z[1] <= z[0] + `ROT0;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[2]   <= 0;
        y[2]   <= 0;
        z[2]   <= 0;
    end
    else if(din_vld_ff[2])begin
        if(z[1][25]==0)begin
            x[2] <= x[1] - (y[1]>>>1);
            y[2] <= y[1] + (x[1]>>>1);
            z[2] <= z[1] - `ROT1;
        end
        else begin
            x[2] <= x[1] + (y[1]>>>1);
            y[2] <= y[1] - (x[1]>>>1);
            z[2] <= z[1] + `ROT1;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[3]   <= 0;
        y[3]   <= 0;
        z[3]   <= 0;
    end
    else if(din_vld_ff[3])begin
        if(z[2][25]==0)begin
            x[3] <= x[2] - (y[2]>>>2);
            y[3] <= y[2] + (x[2]>>>2);
            z[3] <= z[2] - `ROT2;
        end
        else begin
            x[3] <= x[2] + (y[2]>>>2);
            y[3] <= y[2] - (x[2]>>>2);
            z[3] <= z[2] + `ROT2;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[4]   <= 0;
        y[4]   <= 0;
        z[4]   <= 0;
    end
    else if(din_vld_ff[4])begin
        if(z[3][25]==0)begin
            x[4] <= x[3] - (y[3]>>>3);
            y[4] <= y[3] + (x[3]>>>3);
            z[4] <= z[3] - `ROT3;
        end
        else begin
            x[4] <= x[3] + (y[3]>>>3);
            y[4] <= y[3] - (x[3]>>>3);
            z[4] <= z[3] + `ROT3;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[5]   <= 0;
        y[5]   <= 0;
        z[5]   <= 0;
    end
    else if(din_vld_ff[5])begin
        if(z[4][25]==0)begin
            x[5] <= x[4] - (y[4]>>>4);
            y[5] <= y[4] + (x[4]>>>4);
            z[5] <= z[4] - `ROT4;
        end
        else begin
            x[5] <= x[4] + (y[4]>>>4);
            y[5] <= y[4] - (x[4]>>>4);
            z[5] <= z[4] + `ROT4;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[6]   <= 0;
        y[6]   <= 0;
        z[6]   <= 0;
    end
    else if(din_vld_ff[6])begin
        if(z[5][25]==0)begin
            x[6] <= x[5] - (y[5]>>>5);
            y[6] <= y[5] + (x[5]>>>5);
            z[6] <= z[5] - `ROT5;
        end
        else begin
            x[6] <= x[5] + (y[5]>>>5);
            y[6] <= y[5] - (x[5]>>>5);
            z[6] <= z[5] + `ROT5;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[7]   <= 0;
        y[7]   <= 0;
        z[7]   <= 0;
    end
    else if(din_vld_ff[7])begin
        if(z[6][25]==0)begin
            x[7] <= x[6] - (y[6]>>>6);
            y[7] <= y[6] + (x[6]>>>6);
            z[7] <= z[6] - `ROT6;
        end
        else begin
            x[7] <= x[6] + (y[6]>>>6);
            y[7] <= y[6] - (x[6]>>>6);
            z[7] <= z[6] + `ROT6;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[8]   <= 0;
        y[8]   <= 0;
        z[8]   <= 0;
    end
    else if(din_vld_ff[8])begin
        if(z[7][25]==0)begin
            x[8] <= x[7] - (y[7]>>>7);
            y[8] <= y[7] + (x[7]>>>7);
            z[8] <= z[7] - `ROT7;
        end
        else begin
            x[8] <= x[7] + (y[7]>>>7);
            y[8] <= y[7] - (x[7]>>>7);
            z[8] <= z[7] + `ROT7;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[9]   <= 0;
        y[9]   <= 0;
        z[9]   <= 0;
    end
    else if(din_vld_ff[9])begin
        if(z[8][25]==0)begin
            x[9] <= x[8] - (y[8]>>>8);
            y[9] <= y[8] + (x[8]>>>8);
            z[9] <= z[8] - `ROT8;
        end
        else begin
            x[9] <= x[8] + (y[8]>>>8);
            y[9] <= y[8] - (x[8]>>>8);
            z[9] <= z[8] + `ROT8;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[10]   <= 0;
        y[10]   <= 0;
        z[10]   <= 0;
    end
    else if(din_vld_ff[10])begin
        if(z[9][25]==0)begin
            x[10] <= x[9] - (y[9]>>>9);
            y[10] <= y[9] + (x[9]>>>9);
            z[10] <= z[9] - `ROT9;
        end
        else begin
            x[10] <= x[9] + (y[9]>>>9);
            y[10] <= y[9] - (x[9]>>>9);
            z[10] <= z[9] + `ROT9;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[11]   <= 0;
        y[11]   <= 0;
        z[11]   <= 0;
    end
    else if(din_vld_ff[11])begin
        if(z[10][25]==0)begin
            x[11] <= x[10] - (y[10]>>>10);
            y[11] <= y[10] + (x[10]>>>10);
            z[11] <= z[10] - `ROT10;
        end
        else begin
            x[11] <= x[10] + (y[10]>>>10);
            y[11] <= y[10] - (x[10]>>>10);
            z[11] <= z[10] + `ROT10;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[12]   <= 0;
        y[12]   <= 0;
        z[12]   <= 0;
    end
    else if(din_vld_ff[12])begin
        if(z[11][25]==0)begin
            x[12] <= x[11] - (y[11]>>>11);
            y[12] <= y[11] + (x[11]>>>11);
            z[12] <= z[11] - `ROT11;
        end
        else begin
            x[12] <= x[11] + (y[11]>>>11);
            y[12] <= y[11] - (x[11]>>>11);
            z[12] <= z[11] + `ROT11;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[13]   <= 0;
        y[13]   <= 0;
        z[13]   <= 0;
    end
    else if(din_vld_ff[13])begin
        if(z[12][25]==0)begin
            x[13] <= x[12] - (y[12]>>>12);
            y[13] <= y[12] + (x[12]>>>12);
            z[13] <= z[12] - `ROT12;
        end
        else begin
            x[13] <= x[12] + (y[12]>>>12);
            y[13] <= y[12] - (x[12]>>>12);
            z[13] <= z[12] + `ROT12;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[14]   <= 0;
        y[14]   <= 0;
        z[14]   <= 0;
    end
    else if(din_vld_ff[14])begin
        if(z[13][25]==0)begin
            x[14] <= x[13] - (y[13]>>>13);
            y[14] <= y[13] + (x[13]>>>13);
            z[14] <= z[13] - `ROT13;
        end
        else begin
            x[14] <= x[13] + (y[13]>>>13);
            y[14] <= y[13] - (x[13]>>>13);
            z[14] <= z[13] + `ROT13;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[15]   <= 0;
        y[15]   <= 0;
        z[15]   <= 0;
    end
    else if(din_vld_ff[15])begin
        if(z[14][25]==0)begin
            x[15] <= x[14] - (y[14]>>>14);
            y[15] <= y[14] + (x[14]>>>14);
            z[15] <= z[14] - `ROT14;
        end
        else begin
            x[15] <= x[14] + (y[14]>>>14);
            y[15] <= y[14] - (x[14]>>>14);
            z[15] <= z[14] + `ROT14;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        x[16]   <= 0;
        y[16]   <= 0;
        z[16]   <= 0;
    end
    else if(din_vld_ff[16])begin
        if(z[15][25]==0)begin
            x[16] <= x[15] - (y[15]>>>15);
            y[16] <= y[15] + (x[15]>>>15);
            z[16] <= z[15] - `ROT15;
        end
        else begin
            x[16] <= x[15] + (y[15]>>>15);
            y[16] <= y[15] - (x[15]>>>15);
            z[16] <= z[15] + `ROT15;
        end
    end
end

always @(posedge clk or negedge rst_n )begin 
    if(rst_n==0) begin
        din_vld_ff <= (0)  ;
    end
    else begin
        din_vld_ff <= ({din_vld_ff[15:0],din_vld})  ;
    end 
end

always @(posedge clk or negedge rst_n )begin 
    if(rst_n==0) begin
        dout_vld_ff0 <= (0)  ;
        dout_vld     <= (0)  ;
    end
    else begin
        dout_vld_ff0 <= (din_vld_ff[16])  ; //打2拍处理
        dout_vld     <= (dout_vld_ff0)  ;
    end 
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        flag_ff <= 0;
    end
    else begin
        flag_ff <= {flag_ff[33:0],flag};
    end
end

always @(posedge clk or negedge rst_n )begin 
    if(rst_n==0) begin
        dout_sin <= (0)  ;
    end
    else if(flag_ff[33:32]==0)begin       //第一象限，y(16) = sin(x)
        dout_sin <= (y[16])  ;
    end 
    else if(flag_ff[33:32]==1)begin       //第二象限，Sin(X)=Sin(A+90)=CosA,Cos(X)=Cos(A+90)=-SinA
        dout_sin <= (x[16])  ;
    end 
    else if(flag_ff[33:32]==2)begin       //第三象限，the Sin(X)=Sin(A+180)=-SinA,Cos(X)=Cos(A+180)=-CosA
        dout_sin <= ~(y[16]) + 1'b1  ;
    end 
    else if(flag_ff[33:32]==3)begin       //第四象限，the Sin(X)=Sin(A+270)=-CosA,Cos(X)=Cos(A+270)=SinA
        dout_sin <= ~(x[16]) + 1'b1  ;
    end 
end

always @(posedge clk or negedge rst_n )begin 
    if(rst_n==0) begin
        dout_cos <= (0)  ;
    end
    else if(flag_ff[33:32]==0)begin       //第一象限 x(16) = cos(x)
        dout_cos <= (x[16])  ;
    end 
    else if(flag_ff[33:32]==1)begin       //第二象限，Sin(X)=Sin(A+90)=CosA,Cos(X)=Cos(A+90)=-SinA
        dout_cos <= ~(y[16]) + 1'b1  ;
    end 
    else if(flag_ff[33:32]==2)begin       //第三象限，the Sin(X)=Sin(A+180)=-SinA,Cos(X)=Cos(A+180)=-CosA
        dout_cos <= ~(x[16]) + 1'b1  ;
    end
    else if(flag_ff[33:32]==3)begin       //第四象限，the Sin(X)=Sin(A+270)=-CosA,Cos(X)=Cos(A+270)=SinA
        dout_cos <= (y[16])  ;
    end 
end

endmodule
